@@ -152,13 +152,14 @@ pub(crate) fn change_positions(lhs_src: &str, rhs_src: &str) -> Vec<MatchedPos>
                 // have a very large number of words, don't diff
                 // individual words.
                 if lhs_words.len() > MAX_WORDS_IN_LINE || rhs_words.len() > MAX_WORDS_IN_LINE {
-                    let lhs_pos = lhs_lp.from_region(lhs_offset, lhs_offset + lhs_part.len());
-                    mps.push(MatchedPos {
-                        kind: MatchKind::NovelWord {
-                            highlight: TokenKind::Atom(AtomKind::Normal),
-                        },
-                        pos: lhs_pos[0],
-                    });
+                    for lhs_pos in lhs_lp.from_region(lhs_offset, lhs_offset + lhs_part.len()) {
+                        mps.push(MatchedPos {
+                            kind: MatchKind::NovelWord {
+                                highlight: TokenKind::Atom(AtomKind::Normal),
+                            },
+                            pos: lhs_pos,
+                        });
+                    }
 
                     lhs_offset += lhs_part.len();
                     rhs_offset += rhs_part.len();
@@ -170,6 +171,7 @@ pub(crate) fn change_positions(lhs_src: &str, rhs_src: &str) -> Vec<MatchedPos>
                         myers_diff::DiffResult::Left(lhs_word) => {
                             let lhs_pos =
                                 lhs_lp.from_region(lhs_offset, lhs_offset + lhs_word.len());
+
                             mps.push(MatchedPos {
                                 kind: MatchKind::NovelWord {
                                     highlight: TokenKind::Atom(AtomKind::Normal),
