@@ -1,13 +1,12 @@
 fn main() {
     let src_dir = std::path::Path::new("src");
+
     let mut c_config = cc::Build::new();
-    c_config.include(src_dir);
-    c_config
-        .flag_if_supported("-Wno-unused-parameter")
-        .flag_if_supported("-Wno-unused-but-set-variable")
-        .flag_if_supported("-Wno-trigraphs");
+    c_config.std("c11").include(src_dir);
+
     let parser_path = src_dir.join("parser.c");
     c_config.file(&parser_path);
     println!("cargo:rerun-if-changed={}", parser_path.to_str().unwrap());
-    c_config.compile("parser");
+
+    c_config.compile("tree-sitter-go");
 }
