@@ -1,27 +1,20 @@
-// -*- coding: utf-8 -*-
-// ------------------------------------------------------------------------------------------------
-// Copyright © 2020, tree-sitter-go authors.
-// See the LICENSE file in this repo for license details.
-// ------------------------------------------------------------------------------------------------
-
-//! This crate provides a Go grammar for the [tree-sitter][] parsing library.
+//! This crate provides Go language support for the [tree-sitter][] parsing library.
 //!
-//! Typically, you will use the [language][language func] function to add this grammar to a
+//! Typically, you will use the [language][language func] function to add this language to a
 //! tree-sitter [Parser][], and then use the parser to parse some code:
 //!
 //! ```
 //! use tree_sitter::Parser;
 //!
 //! let code = r#"
-//!     func double(x int) int {
-//!         return x * 2
-//!     }
+//! func add(a, b int) int {
+//!     return a + b
+//! }
 //! "#;
 //! let mut parser = Parser::new();
-//! parser.set_language(tree_sitter_go::language()).expect("Error loading Go grammar");
-//! let parsed = parser.parse(code, None).unwrap();
-//! let root = parsed.root_node();
-//! assert!(!root.has_error());
+//! parser.set_language(&tree_sitter_go::language()).expect("Error loading Go grammar");
+//! let tree = parser.parse(code, None).unwrap();
+//! assert!(!tree.root_node().has_error());
 //! ```
 //!
 //! [Language]: https://docs.rs/tree-sitter/*/tree_sitter/struct.Language.html
@@ -35,34 +28,31 @@ extern "C" {
     fn tree_sitter_go() -> Language;
 }
 
-/// Returns the tree-sitter [Language][] for this grammar.
+/// Get the tree-sitter [Language][] for this grammar.
 ///
 /// [Language]: https://docs.rs/tree-sitter/*/tree_sitter/struct.Language.html
 pub fn language() -> Language {
     unsafe { tree_sitter_go() }
 }
 
-/// The source of the Go tree-sitter grammar description.
-pub const GRAMMAR: &str = include_str!("../../grammar.js");
-
-/// The syntax highlighting query for this language.
-pub const HIGHLIGHT_QUERY: &str = include_str!("../../queries/highlights.scm");
-
 /// The content of the [`node-types.json`][] file for this grammar.
 ///
 /// [`node-types.json`]: https://tree-sitter.github.io/tree-sitter/using-parsers#static-node-types
 pub const NODE_TYPES: &str = include_str!("../../src/node-types.json");
 
+/// The syntax highlighting query for this language.
+pub const HIGHLIGHTS_QUERY: &str = include_str!("../../queries/highlights.scm");
+
 /// The symbol tagging query for this language.
-pub const TAGGING_QUERY: &str = include_str!("../../queries/tags.scm");
+pub const TAGS_QUERY: &str = include_str!("../../queries/tags.scm");
 
 #[cfg(test)]
 mod tests {
     #[test]
-    fn can_load_grammar() {
+    fn test_can_load_grammar() {
         let mut parser = tree_sitter::Parser::new();
         parser
-            .set_language(super::language())
+            .set_language(&super::language())
             .expect("Error loading Go grammar");
     }
 }
